#ifndef _RH_IXGBE_BACKPORT_COMPAT_H_
#define _RH_IXGBE_BACKPORT_COMPAT_H_

#include <linux/netdevice.h>
#include <linux/pci.h>
#include <linux/skbuff.h>
#include <net/checksum.h>
#include <net/udp_tunnel.h>

/* legacy drivers only, netdev_start_xmit() sets txq->trans_start */
static inline void netif_trans_update(struct net_device *dev)
{
	dev->trans_start = jiffies;
}


#define NETIF_F_TSO_MANGLEID 0
#define NETIF_F_GSO_PARTIAL  0


/*
 * hlist_add_behind() is a new name for hlist_add_after(), with reversed
 * argument order.
 */
#define hlist_add_behind(a, b) hlist_add_after(b, a)

/* -- List of parsable UDP tunnel types --
 *
 * Adding to this list will result in serious debate.  The main issue is
 * that this list is essentially a list of workarounds for either poorly
 * designed tunnels, or poorly designed device offloads.
 *
 * The parsing supported via these types should really be used for Rx
 * traffic only as the network stack will have already inserted offsets for
 * the location of the headers in the skb.  In addition any ports that are
 * pushed should be kept within the namespace without leaking to other
 * devices such as VFs or other ports on the same device.
 *
 * It is strongly encouraged to use CHECKSUM_COMPLETE for Rx to avoid the
 * need to use this for Rx checksum offload.  It should not be necessary to
 * call this function to perform Tx offloads on outgoing traffic.
 */
enum udp_parsable_tunnel_type {
	UDP_TUNNEL_TYPE_VXLAN,		/* RFC 7348 */
	UDP_TUNNEL_TYPE_GENEVE,		/* draft-ietf-nvo3-geneve */
};


struct udp_tunnel_info {
	unsigned short type;
	sa_family_t sa_family;
	__be16 port;
};


static inline void udp_tunnel_get_rx_info(struct net_device *dev)
{
	ASSERT_RTNL();
	call_netdevice_notifiers(NETDEV_OFFLOAD_PUSH_VXLAN, dev);
	call_netdevice_notifiers(NETDEV_OFFLOAD_PUSH_GENEVE, dev);
}


static inline unsigned char *skb_checksum_start(const struct sk_buff *skb)
{
	return skb->head + skb->csum_start;
}


static inline int
pci_request_mem_regions(struct pci_dev *pdev, const char *name)
{
	return pci_request_selected_regions(pdev,
			pci_select_bars(pdev, IORESOURCE_MEM), name);
}

static inline void
pci_release_mem_regions(struct pci_dev *pdev)
{
	return pci_release_selected_regions(pdev,
			pci_select_bars(pdev, IORESOURCE_MEM));
}


static inline void csum_replace_by_diff(__sum16 *sum, __wsum diff)
{
	*sum = csum_fold(csum_add(diff, ~csum_unfold(*sum)));
}

#endif /* #ifndef _RH_IXGBE_BACKPORT_COMPAT_H_ */
